# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file.
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

"""
This module customizes the default Django database creation for Aurora DSQL.
In order to customize the database creation process, the module overrides
certain functions with custom logic.
"""

from django.db.backends.postgresql import creation


class DatabaseCreation(creation.DatabaseCreation):
    def _clone_test_db(self, suffix, verbosity, keepdb=False):
        raise NotImplementedError(
            "Aurora DSQL doesn't support cloning databases. Disable the option to run tests in parallel processes."
        )



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file.
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

"""
This module customizes the default Django database schema editor functions
for Aurora DSQL.
"""

from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.backends.postgresql import schema
from django.db.models import CheckConstraint


class DatabaseSchemaEditor(schema.DatabaseSchemaEditor):
    """
    Aurora DSQL schema editor based on the PostgreSQL backend.

    Aurora DSQL is PostgreSQL-compatible but supports a subset of PostgreSQL
    operations. This class overrides SQL templates and methods to work within
    DSQL's constraints.
    """

    # Use DSQL's async index creation syntax.
    sql_create_index = "CREATE INDEX ASYNC %(name)s ON %(table)s%(using)s (%(columns)s)%(include)s%(extra)s%(condition)s"

    # Create unique constraints as unique indexes instead of using "ALTER TABLE".
    sql_create_unique = "CREATE UNIQUE INDEX ASYNC %(name)s ON %(table)s (%(columns)s)"

    # Delete unique constraints by dropping the underlying index.
    sql_delete_unique = "DROP INDEX %(name)s CASCADE"

    # Remove constraint management from default updates.
    sql_update_with_default = "UPDATE %(table)s SET %(column)s = %(default)s WHERE %(column)s IS NULL"

    def __enter__(self):
        super().__enter__()
        # As long as DatabaseFeatures.can_rollback_ddl = False, compose() may
        # fail if connection is None as per
        # https://github.com/django/django/pull/15687#discussion_r1038175823.
        # See also
        # https://github.com/django/django/pull/15687#discussion_r1041503991.
        self.connection.ensure_connection()
        return self

    def add_index(self, model, index, concurrently=False):
        if index.contains_expressions and not self.connection.features.supports_expression_indexes:
            return
        super().add_index(model, index, concurrently)

    def remove_index(self, model, index, concurrently=False):
        if index.contains_expressions and not self.connection.features.supports_expression_indexes:
            return
        super().remove_index(model, index, concurrently)

    def _check_sql(self, name, check):
        # There is no feature check in the upstream implementation when creating
        # a model, so we add our own check.
        if not self.connection.features.supports_table_check_constraints:
            return None
        return super()._check_sql(name, check)

    def add_constraint(self, model, constraint):
        # Older versions of Django don't reference supports_table_check_constraints, so we add this as a backup.
        if isinstance(constraint, CheckConstraint) and not self.connection.features.supports_table_check_constraints:
            return
        super().add_constraint(model, constraint)

    def remove_constraint(self, model, constraint):
        # Older versions of Django don't reference supports_table_check_constraints, so we add this as a backup.
        if isinstance(constraint, CheckConstraint) and not self.connection.features.supports_table_check_constraints:
            return
        super().remove_constraint(model, constraint)

    def _index_columns(self, table, columns, col_suffixes, opclasses):
        # Aurora DSQL doesn't support PostgreSQL opclasses.
        return BaseDatabaseSchemaEditor._index_columns(self, table, columns, col_suffixes, opclasses)

    def _create_like_index_sql(self, model, field):
        # Aurora DSQL doesn't support LIKE indexes which use postgres
        # opsclasses
        return None



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
# and limitations under the License.


# Create your tests here.



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file.
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

"""
A module with custom wrapper that overrides base postgres database operations
adapter in order to make it work with Aurora DSQL.
"""

from django.db.backends.postgresql import operations


class DatabaseOperations(operations.DatabaseOperations):
    cast_data_types = {
        "AutoField": "uuid",
        "BigAutoField": "uuid",
        "SmallAutoField": "smallint",
    }

    def deferrable_sql(self):
        # Deferrable constraints aren't supported:
        return ""

    def integer_field_range(self, internal_type):
        """
        Override to handle UUIDField which doesn't have integer ranges.
        """
        if internal_type == "UUIDField":
            # Skip validation.
            return None, None
        return super().integer_field_range(internal_type)



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

import uuid

from django.db import models

# Create your models here.


class Owner(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=30, blank=False)
    # This is many to one relation
    city = models.CharField(max_length=80, blank=False)
    telephone = models.CharField(max_length=20, blank=True, null=True, default=None)

    def __str__(self):
        return f"{self.name}"


class Pet(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=30, blank=False)
    birth_date = models.DateField()
    owner = models.ForeignKey(Owner, on_delete=models.CASCADE, db_constraint=False, null=True)


class Specialty(models.Model):
    name = models.CharField(max_length=80, blank=False, primary_key=True)

    def __str__(self):
        return self.name


class Vet(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=30, blank=False)
    specialties = models.ManyToManyField(Specialty, through="VetSpecialties")
    owner = models.OneToOneField(Owner, on_delete=models.SET_DEFAULT, db_constraint=False, null=True, blank=True, default=None)

    def __str__(self):
        return f"{self.name}"


# Need to use custom intermediate table because Django considers default primary
# keys as integers. We use UUID as default primary key which is not an integer.
class VetSpecialties(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    vet = models.ForeignKey(Vet, on_delete=models.CASCADE, db_constraint=False)
    specialty = models.ForeignKey(Specialty, on_delete=models.CASCADE, db_constraint=False)


class Visits(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    pet = models.ForeignKey(Pet, on_delete=models.CASCADE, db_constraint=False)
    vet = models.ForeignKey(Vet, on_delete=models.CASCADE, db_constraint=False)
    visit_date = models.DateField()
    description = models.TextField()



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file.
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

"""
This is a wrapper around the Django's base postgres database API.
In case of Aurora DSQL, password is a SigV4 token which must be rotated every
N seconds. This module extends the base wrapper to handle this case.
"""

import logging
import uuid

import boto3
from botocore.exceptions import BotoCoreError, ClientError
from django.core.exceptions import ValidationError
from django.db import models
from django.db.backends.postgresql import base
from django.db.models.fields import Field
from django.utils.translation import gettext_lazy

from .creation import DatabaseCreation
from .features import DatabaseFeatures
from .operations import DatabaseOperations
from .schema import DatabaseSchemaEditor

logger = logging.getLogger(__name__)


def get_aws_connection_params(params):
    """
    Get connection parameters to establish a connection to the Aurora DSQL
    cluster. Uses AWS SDK to generate the password token and sets it in the
    connection parameters.

    Args:
        params (dict): Parameter dictionary that client sets in DATABASES variable

    Returns:
        dict: returns a dictionary of connection parameters
    """
    region = params.pop("region", None)
    hostname = params.get("host")
    user = params.get("user")
    sslmode = params.get("sslmode", None)
    sslrootcert = params.get("sslrootcert", None)
    if sslrootcert is None and sslmode == "verify-full":
        params["sslrootcert"] = "system"
    expires_in = params.pop("expires_in", None)
    aws_profile = params.pop("aws_profile", None)

    try:
        session = boto3.session.Session(profile_name=aws_profile) if aws_profile else boto3.session.Session()
        client = session.client("dsql", region_name=region)

        # Set correct IAM Auth token
        is_admin = user == "admin"
        has_expires_in = expires_in is not None
        if is_admin and has_expires_in:
            params["password"] = client.generate_db_connect_admin_auth_token(hostname, region, expires_in)
        elif is_admin and not has_expires_in:
            params["password"] = client.generate_db_connect_admin_auth_token(hostname, region)
        elif not is_admin and has_expires_in:
            params["password"] = client.generate_db_connect_auth_token(hostname, region, expires_in)
        else:
            params["password"] = client.generate_db_connect_auth_token(hostname, region)

        params.setdefault("port", 5432)

    except (BotoCoreError, ClientError) as e:
        logger.error("Failed to generate DB auth token: %s", str(e))
        raise

    return params


class DatabaseWrapper(base.DatabaseWrapper):
    """
    A wrapper class that adapts the Django base API for Aurora DSQL
    """

    vendor = "dsql"
    display_name = "Aurora DSQL"
    # Override some types from the postgresql adapter.
    data_types = dict(
        base.DatabaseWrapper.data_types,
        BigAutoField="uuid",
        AutoField="uuid",
        DateTimeField="timestamptz",
    )
    data_types_suffix = dict(
        base.DatabaseWrapper.data_types_suffix,
        BigAutoField="DEFAULT gen_random_uuid()",
        # For now skipping small int because uuid does not fit in a smallint?
        SmallAutoField="",
        AutoField="DEFAULT gen_random_uuid()",
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._patch_autofields()

        # Automatically disable server-side cursors since Aurora DSQL doesn't support them.
        # We preserve the user config if it is defined but this is likely a user mistake.
        self.settings_dict.setdefault("DISABLE_SERVER_SIDE_CURSORS", True)

    def _patch_autofields(self):
        """
        Patch AutoField classes to return UUID type for related fields.
        This ensures ForeignKey fields that reference AutoFields are also UUIDs.
        """

        def uuid_rel_db_type(self, connection):
            return "uuid"

        def uuid_get_prep_value(self, value):
            """Override get_prep_value to prevent int() conversion of UUIDs."""
            return Field.get_prep_value(self, value)

        def uuid_to_python(self, value):
            """Convert provided value to a UUID where possible."""
            if value is None or isinstance(value, uuid.UUID):
                return value
            if isinstance(value, str):
                try:
                    return uuid.UUID(value)
                except ValueError:
                    pass

            raise ValidationError(
                self.error_messages["invalid"],
                code="invalid",
                params={"value": value},
            )

        for field_class in [models.AutoField, models.BigAutoField]:
            field_class.rel_db_type = uuid_rel_db_type
            field_class.get_prep_value = uuid_get_prep_value
            field_class.to_python = uuid_to_python
            field_class.default_error_messages = {
                "invalid": gettext_lazy("'%(value)s' value must be a valid UUID."),
            }

    SchemaEditorClass = DatabaseSchemaEditor
    creation_class = DatabaseCreation
    features_class = DatabaseFeatures
    ops_class = DatabaseOperations

    def get_connection_params(self):
        params = super().get_connection_params()
        return get_aws_connection_params(params)

    def check_constraints(self, table_names=None):
        """
        Override to do nothing since SET CONSTRAINTS is not supported.
        """

    def disable_constraint_checking(self):
        """
        Override to do nothing since SET CONSTRAINTS is not supported.
        """
        return True

    def enable_constraint_checking(self):
        """
        Override to do nothing since SET CONSTRAINTS is not supported.
        """



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

import html
import json
import time

from django.db import Error
from django.db.transaction import atomic
from django.http import HttpResponse, HttpResponseBadRequest, JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from pet_clinic.models import Owner, Pet, Specialty, Vet, VetSpecialties

# Create your views here.


##
# initial_wait is the amount of time after with the operation is retried
# delay_factor is the pace at which the retries slow down upon each failure.
# For example an initial_wait of 1 and delay_factor of 2 implies,
# First retry occurs after 1 second, second one after 1*2 = 2 seconds,
# Third one after 2*2 = 4 seconds, forth one after 4*2 = 8 seconds and so on.
##
def with_retries(retries=3, failed_response=HttpResponse(status=500), initial_wait=1, delay_factor=2):
    def handle(view):
        def retry_fn(*args, **kwargs):
            delay = initial_wait
            for i in range(retries):
                print(("attempt: %s/%s") % (i + 1, retries))
                try:
                    return view(*args, **kwargs)
                # TODO: check error code?
                except Error as e:
                    print(f"Error: {e}, retrying...")
                    time.sleep(delay)
                    delay *= delay_factor
            return failed_response

        return retry_fn

    return handle


@method_decorator(csrf_exempt, name="dispatch")
class OwnerView(View):
    @with_retries()
    def get(self, request, id=None, *args, **kwargs):
        owners = Owner.objects
        # Apply filter if specific id is requested.
        if id is not None:
            owners = owners.filter(id=id)
        return JsonResponse(list(owners.values()), safe=False)

    @with_retries()
    @atomic
    def post(self, request, *args, **kwargs):
        data = json.loads(request.body.decode())

        # If id is provided we try updating the existing object
        id = data.get("id", None)
        try:
            owner = Owner.objects.get(id=id) if id is not None else None
        except Exception:
            return HttpResponseBadRequest(("error: check if owner with id `%s` exists") % (html.escape(str(id))))

        name = data.get("name", owner.name if owner else None)
        # Either the name or id must be provided.
        if owner is None and name is None:
            return HttpResponseBadRequest()

        telephone = data.get("telephone", owner.telephone if owner else None)
        city = data.get("city", owner.city if owner else None)

        if owner is None:
            # Owner _not_ present, creating new one
            print(("owner: %s is not present; adding") % (name))
            owner = Owner(name=name, telephone=telephone, city=city)
        else:
            # Owner present, update existing
            print(("owner: %s is present; updating") % (name))
            owner.name = name
            owner.telephone = telephone
            owner.city = city

        owner.save()
        return JsonResponse(list(Owner.objects.filter(id=owner.id).values()), safe=False)

    @with_retries()
    @atomic
    def delete(self, request, id=None, *args, **kwargs):
        if id is not None:
            Owner.objects.filter(id=id).delete()
        return HttpResponse(status=200)


@method_decorator(csrf_exempt, name="dispatch")
class PetView(View):
    @with_retries()
    def get(self, request=None, id=None, *args, **kwargs):
        pets = Pet.objects
        # Apply filter if specific id is requested.
        if id is not None:
            pets = pets.filter(id=id)
        return JsonResponse(list(pets.values()), safe=False)

    @with_retries()
    @atomic
    def post(self, request, *args, **kwargs):
        data = json.loads(request.body.decode())

        # If id is provided we try updating the existing object
        id = data.get("id", None)
        try:
            pet = Pet.objects.get(id=id) if id is not None else None
        except Exception:
            return HttpResponseBadRequest(("error: check if pet with id `%s` exists") % (html.escape(str(id))))

        name = data.get("name", pet.name if pet else None)
        # Either the name or id must be provided.
        if pet is None and name is None:
            return HttpResponseBadRequest()

        birth_date = data.get("birth_date", pet.birth_date if pet else None)
        owner_id = data.get("owner_id", pet.owner.id if pet and pet.owner else None)
        try:
            owner = Owner.objects.get(id=owner_id) if owner_id else None
        except Exception:
            return HttpResponseBadRequest(("error: check if owner with id `%s` exists") % (html.escape(str(owner_id))))

        if pet is None:
            # Pet _not_ present, creating new one
            print(("pet name: %s is not present; adding") % (name))
            pet = Pet(name=name, birth_date=birth_date, owner=owner)
        else:
            # Pet present, update existing
            print(("pet name: %s is present; updating") % (name))
            pet.name = name
            pet.birth_date = birth_date
            pet.owner = owner

        pet.save()
        return JsonResponse(list(Pet.objects.filter(id=pet.id).values()), safe=False)

    @with_retries()
    @atomic
    def delete(self, request=None, id=None, *args, **kwargs):
        if id is not None:
            Pet.objects.filter(id=id).delete()
        return HttpResponse(status=200)


@method_decorator(csrf_exempt, name="dispatch")
class VetView(View):
    @with_retries()
    def get(self, request=None, id=None, *args, **kwargs):
        vets = Vet.objects
        # Apply filter if specific id is requested.
        if id is not None:
            vets = vets.filter(id=id)
        return JsonResponse(list(vets.values()), safe=False)

    @with_retries()
    @atomic
    def post(self, request, *args, **kwargs):
        data = json.loads(request.body.decode())
        # If id is provided we try updating the existing object
        id = data.get("id", None)
        try:
            vet = Vet.objects.get(id=id) if id is not None else None
        except Exception:
            return HttpResponseBadRequest(("error: check if vet with id `%s` exists") % (html.escape(str(id))))

        name = data.get("name", vet.name if vet else None)

        # Either the name or id must be provided.
        if vet is None and name is None:
            return HttpResponseBadRequest()

        owner_id = data.get("owner_id", vet.owner.id if vet and vet.owner else None)
        try:
            owner = Owner.objects.get(id=owner_id) if owner_id else None
        except Exception:
            return HttpResponseBadRequest(("error: check if owner with id `%s` exists") % (html.escape(str(id))))

        specialties_list = data.get("specialties", vet.specialties if vet and vet.specialties else [])
        specialties = []
        for specialty in specialties_list:
            try:
                specialties_obj = Specialty.objects.get(name=specialty)
            except Exception:
                return HttpResponseBadRequest(("error: check if specialty `%s` exists") % (html.escape(str(specialty))))
            specialties.append(specialties_obj)

        if vet is None:
            print(("vet name: %s, not present, adding") % (name))
            vet = Vet(name=name, owner_id=owner_id)
        else:
            print(("vet name: %s, present, updating") % (name))
            vet.name = name
            vet.owner = owner

        # First save the vet so that we have an id. Then we can add specialties.
        # Django needs the id primary key of the parent object before adding relations
        vet.save()

        # Add any specialties provided
        vet.specialties.add(*specialties)
        return JsonResponse(
            {
                "Veterinarian": list(Vet.objects.filter(id=vet.id).values()),
                "Specialties": list(VetSpecialties.objects.filter(vet=vet.id).values()),
            },
            safe=False,
        )

    @with_retries()
    @atomic
    def delete(self, request, id=None, *args, **kwargs):
        if id is not None:
            Vet.objects.filter(id=id).delete()
        return HttpResponse(status=200)


@method_decorator(csrf_exempt, name="dispatch")
class SpecialtyView(View):
    @with_retries()
    def get(self, request=None, name=None, *args, **kwargs):
        specialties = Specialty.objects
        # Apply filter if specific name is requested.
        if name is not None:
            specialties = specialties.filter(name=name)
        return JsonResponse(list(specialties.values()), safe=False)

    @with_retries()
    @atomic
    def post(self, request=None, *args, **kwargs):
        data = json.loads(request.body.decode())
        name = data.get("name", None)
        if name is None:
            return HttpResponseBadRequest()
        specialty = Specialty(name=name)
        specialty.save()
        return JsonResponse(list(Specialty.objects.filter(name=specialty.name).values()), safe=False)

    @with_retries()
    @atomic
    def delete(self, request=None, name=None, *args, **kwargs):
        if id is not None:
            Specialty.objects.filter(name=name).delete()
        return HttpResponse(status=200)


@method_decorator(csrf_exempt, name="dispatch")
class VetSpecialtiesView(View):
    @with_retries()
    def get(self, request=None, *args, **kwargs):
        data = json.loads(request.body.decode())
        vet_id = data.get("vet_id", None)
        specialty_id = data.get("specialty_id", None)
        specialties = VetSpecialties.objects
        # Apply filter if specific name is requested.
        if vet_id is not None:
            specialties = specialties.filter(vet_id=vet_id)
        if specialty_id is not None:
            specialties = specialties.filter(specialty_id=specialty_id)
        return JsonResponse(list(specialties.values()), safe=False)



#!/usr/bin/env python

# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""Django's command-line utility for administrative tasks."""

import os
import sys


def main():
    """Run administrative tasks."""
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "project.settings")
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    execute_from_command_line(sys.argv)


if __name__ == "__main__":
    main()



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file.
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

"""
A module with custom wrapper that overrides base postgres database features
adapter in order to make it work with Aurora DSQL. Certain Django features can
be enabled and disabled using this adapter.
"""

from django.db.backends.postgresql import features


class DatabaseFeatures(features.DatabaseFeatures):
    # Can run a DDL inside a transaction.
    # If true, multiple DDL statement are run in same transaction.
    can_rollback_ddl = False

    # Can a fixture contain forward references? i.e., are
    # FK constraints checked at the end of transaction, or
    # at the end of each save operation?
    supports_forward_references = True

    supports_foreign_keys = False

    supports_table_check_constraints = False

    # Can it create foreign key constraints inline when adding columns?
    can_create_inline_fk = False

    # Can the backend clone databases for parallel test execution?
    # Defaults to False to allow third-party backends to opt-in.
    can_clone_databases = False

    # Can constraint checks be deferred until the end of a transaction?
    can_defer_constraint_checks = False

    # Does the database support deferrable unique constraints?
    supports_deferrable_unique_constraints = False

    # Does the database have native JSON field support?
    has_native_json_field = False

    # Can the database introspect materialized views?
    can_introspect_materialized_views = False

    # Can the database rename an index?
    can_rename_index = True

    supports_expression_indexes = False

    # Does the database use savepoints for nested transactions?
    uses_savepoints = False

    # Can savepoints be released, allowing partial rollback of nested
    # transactions?
    can_release_savepoints = False



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
URL configuration for project project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.urls import path
from pet_clinic.views import OwnerView, PetView, SpecialtyView, VetSpecialtiesView, VetView

urlpatterns = [
    path("owner/", OwnerView.as_view(), name="owner"),
    path("owner/<id>", OwnerView.as_view(), name="owner"),
    path("pet/", PetView.as_view(), name="pet"),
    path("pet/<id>", PetView.as_view(), name="pet"),
    path("vet/", VetView.as_view(), name="vet"),
    path("vet/<id>", VetView.as_view(), name="vet"),
    path("specialty/", SpecialtyView.as_view(), name="specialty"),
    path("specialty/<name>", SpecialtyView.as_view(), name="specialty"),
    path("vet-specialties/", VetSpecialtiesView.as_view(), name="vet-specialties"),
]



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
# the License. A copy of the License is located at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
Django settings for project project.

Generated by 'django-admin startproject' using Django 5.1.2.

For more information on this file, see
https://docs.djangoproject.com/en/5.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.1/ref/settings/
"""

from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.1/howto/deployment/checklist/

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]


# Application definition

INSTALLED_APPS = ["pet_clinic"]

MIDDLEWARE = []

ROOT_URLCONF = "project.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
            ],
        },
    },
]

WSGI_APPLICATION = "project.wsgi.application"

SECRET_KEY = "foobar_this_is_not_relevant_and_unused"  # nosec

# Database
# https://docs.djangoproject.com/en/5.1/ref/settings/#databases

DATABASES = {
    "default": {
        "HOST": "luabttng4i6jdbng53eorltjc4.dsql-gamma.us-east-1.on.aws",
        "USER": "admin",
        "NAME": "postgres",
        "ENGINE": "aurora_dsql_django",
        "OPTIONS": {"sslmode": "require", "region": "us-east-1"},
    }
}


# Password validation
# https://docs.djangoproject.com/en/5.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.1/howto/static-files/

STATIC_URL = "static/"

# Default primary key field type
# https://docs.djangoproject.com/en/5.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
